/*
 * Routine that "sparsifies" a vector to a tree.
 * Greedily chooses the largest child of the current nodes.
 *
 * Written by Eric Price, MIT, 2010.
 */

#ifndef SPARSIFY_TREE_H
#define SPARISFY_TREE_H

#include <math.h>
#include <stdio.h>
#include "absvalheap.h"
#include "randomized_select.h"

/*
 * Find a tree of K elements that approximates a given vector.
 * Performs a top-down greedy, so it is not necessarily the best tree.
 */
void sparsify_tree(double *z, int N, int K)
{
  int nlevels;
  int levels[15];
  int i, j, lvl, num, i2, x, y, x2, y2;
  double val;
  double *temp;
  abs_val_heap_t Uheap;

    levels[0] = 0;
    for(nlevels=0; levels[nlevels] < N; nlevels++){
      levels[nlevels+1] = levels[nlevels] + (1<<(2*nlevels));
    }
    if (levels[nlevels] != N){
      printf("Warning: not an even level set\n");
    }

    if (K == N)
        return;
    
    if (K == 0)
    {
        memset(z, 0, N * sizeof(double));
        return;
    }

    
    AbsValHeapCreate(&Uheap, N, 0);
    temp = (double *) calloc(N, sizeof(double));
    AbsValHeapAdd(&Uheap, 0, z[0]);
    for(num = 0; num < K; num++){
      AbsValHeapGetTop(&Uheap, &i, &val);
      temp[i] = val;
      AbsValHeapRemoveTop(&Uheap);
      for(lvl = 0; levels[lvl] <= i; lvl++);lvl--;
      if (lvl == nlevels-1)
        continue;
      x = (i - levels[lvl]) % (1<<lvl);
      y = (i - levels[lvl]) / (1<<lvl);
      for(j = 0; j < 4; j++){
        x2 = (2*x + j%2);
        y2 = (2*y + j/2);
        i2 = levels[lvl+1] + (y2<<(lvl+1)) + x2;
        AbsValHeapAdd(&Uheap, i2, z[i2]);
      }
    }
    for(i = 0; i < N; i++)
      z[i] = temp[i];
    free(temp);
    AbsValHeapDestroy(&Uheap);
}

#endif
